@@ -70,13 +70,13 @@ class ClientTest(TestCase):
         self.assertEqual(response.context['data'], '37')
         self.assertEqual(response.template.name, 'POST Template')
         self.failUnless('Data received' in response.content)
-    
+
     def test_response_headers(self):
         "Check the value of HTTP headers returned in a response"
         response = self.client.get("/test_client/header_view/")
-        
+
         self.assertEquals(response['X-DJANGO-TEST'], 'Slartibartfast')
-        
+
     def test_raw_post(self):
         "POST raw data (with a content type) to a view"
         test_doc = """<?xml version="1.0" encoding="utf-8"?><library><book><title>Blink</title><author>Malcolm Gladwell</author></book></library>"""
@@ -118,6 +118,12 @@ class ClientTest(TestCase):
         # Check that the response was a 301 (permanent redirect) with absolute URI
         self.assertRedirects(response, 'http://django.testserver/test_client/get_view/', status_code=301)
 
+    def test_temporary_redirect(self):
+        "GET a URL that does a non-permanent redirect"
+        response = self.client.get('/test_client/temporary_redirect_view/')
+        # Check that the response was a 302 (non-permanent redirect)
+        self.assertRedirects(response, 'http://testserver/test_client/get_view/', status_code=302)
+
     def test_redirect_to_strange_location(self):
         "GET a URL that redirects to a non-200 page"
         response = self.client.get('/test_client/double_redirect_view/')
@@ -126,6 +132,12 @@ class ClientTest(TestCase):
         # the attempt to get the redirection location returned 301 when retrieved
         self.assertRedirects(response, 'http://testserver/test_client/permanent_redirect_view/', target_status_code=301)
 
+    def test_follow_redirect(self):
+        "A URL that redirects can be followed to termination."
+        response = self.client.get('/test_client/double_redirect_view/', follow=True)
+        self.assertRedirects(response, 'http://testserver/test_client/get_view/', status_code=302, target_status_code=200)
+        self.assertEquals(len(response.redirect_chain), 2)
+
     def test_notfound_response(self):
         "GET a URL that responds as '404:Not Found'"
         response = self.client.get('/test_client/bad_view/')
@@ -411,4 +423,3 @@ class ClientTest(TestCase):
         self.assertEqual(mail.outbox[1].from_email, 'from@example.com')
         self.assertEqual(mail.outbox[1].to[0], 'second@example.com')
         self.assertEqual(mail.outbox[1].to[1], 'third@example.com')
-
